/*
 *  Copyright (C) 2003 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <stdlib.h>
#include <jack/jack.h>

#include "audio.h"

buffer_t *buffers = NULL;
unsigned int buffer_size = 0;
unsigned int num_buffers = 0;
unsigned int play_pos = 0;

float gain_inc = 0.01f;

jack_client_t *client;
jack_port_t *output_port;

int process(jack_nframes_t nframes, void *arg)
{
	unsigned int i,j;
	sample_t *out = (sample_t *)jack_port_get_buffer(output_port,
			nframes);

	for (j=0; j<nframes; j++) {
		out[j] = 0.0f;
	}
	for (i=0; i<num_buffers; i++) {
		if (buffers[i].state == on) {
			for (j=0; j<nframes; j++) {
				out[j] += buffers[i].data[(play_pos + j) %
					buffer_size];
			}
		} else if (buffers[i].state == ramp_up) {
			buffers[i].gain += gain_inc;
			if (buffers[i].gain >= 1.0f) {
				buffers[i].state = on;
				buffers[i].gain = 1.0f;
			}
			for (j=0; j<nframes; j++) {
				out[j] += buffers[i].data[(play_pos + j) %
					buffer_size] * buffers[i].gain;
			}
		} else if (buffers[i].state == ramp_down) {
			buffers[i].gain -= gain_inc;
			if (buffers[i].gain <= 0.0f) {
				buffers[i].state = off;
				buffers[i].gain = 0.0f;
			}
			for (j=0; j<nframes; j++) {
				out[j] += buffers[i].data[(play_pos + j) %
					buffer_size] * buffers[i].gain;
			}
		}
	}
	play_pos = (play_pos + nframes) % buffer_size;

	return 0;
}

/* vi:set ts=8 sts=4 sw=4: */
