/*
 *  Copyright (C) 2003 Steve Harris
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <alsa/asoundlib.h>
#include <sndfile.h>

#include "audio.h"

//#define DEBUG

void midi_action(snd_seq_t * seq_handle);

jack_client_t *client;
jack_port_t *output_port;

int main(int argc, char **argv)
{
    snd_seq_t *seq_handle;
    int npfd;
    struct pollfd *pfd;
    int portid_in;
    //int portid_out;
    //snd_midi_event_t *midi_parser;
    //snd_seq_event_t seq_pulse;
    unsigned int i;
    int count;


    if (argc < 2) {
	    fprintf(stderr, "Usage: %s <audio-file> ...\n", argv[0]);
	    return 1;
    }

    /* JACK stuff */

    if ((client = jack_client_new("clock-loop")) == 0) {
	fprintf (stderr, "jack server not running?\n");
	return 1;
    }
    jack_set_process_callback (client, process, 0);
    output_port = jack_port_register (client, "out", JACK_DEFAULT_AUDIO_TYPE,
		    JackPortIsOutput, 0);

    num_buffers = argc - 1;
    buffers = malloc(sizeof(buffer_t) * num_buffers);
    buffer_size = 0;
    for (i = 0; i<num_buffers; i++) {
	    SNDFILE *sf;
	    SF_INFO sfi;

	    sfi.format = 0;
	    sf = sf_open(argv[i + 1], SFM_READ, &sfi);
	    if (sfi.channels != 1) {
		    fprintf(stderr, "%s: only works with mono files\n", argv[0]);
		    return 1;
	    }
	    if (!buffer_size) {
		    buffer_size = sfi.frames;
		    gain_inc = (float)jack_get_buffer_size(client) /
			    (buffer_size * 8.0f);
		    buffers[i].state = on;
		    buffers[i].gain = 1.0f;
	    } else {
		    buffers[i].state = off;
		    buffers[i].gain = 0.0f;
	    }
	    if (sfi.frames > buffer_size) {
		    fprintf(stderr, "%s warning: '%s' is %d frames longer than reference file, truncating\n", argv[0], argv[i+1], (int)(sfi.frames - buffer_size));
	    }
	    buffers[i].data = malloc(buffer_size * sizeof(float));
	    count = sf_read_float(sf, buffers[i].data, buffer_size);
	    if (count < buffer_size) {
		    fprintf(stderr, "%s warning: only read %d/%d frames from file '%s', zero padding\n", argv[0], count, buffer_size, argv[i+1]);
		    memset(buffers[i].data + count, 0, buffer_size - count);
	    }
	    sf_close(sf);
    }
	    
    if (snd_seq_open(&seq_handle, "default", SND_SEQ_OPEN_INPUT, 0) < 0) {
	fprintf(stderr, "Error opening ALSA sequencer.\n");
	exit(1);
    }
    snd_seq_set_client_name(seq_handle, "ClockLoop");
    if ((portid_in =
	 snd_seq_create_simple_port(seq_handle, "in",
				    SND_SEQ_PORT_CAP_WRITE |
				    SND_SEQ_PORT_CAP_SUBS_WRITE,
				    SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
	fprintf(stderr, "Error creating sequencer port.\n");
	exit(1);
    }
    /*
    if ((portid_out =
	 snd_seq_create_simple_port(seq_handle, "out",
				    SND_SEQ_PORT_CAP_READ |
				    SND_SEQ_PORT_CAP_SUBS_READ,
				    SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
	fprintf(stderr, "Error creating sequencer port.\n");
	exit(1);
    }
    */

    if (jack_activate(client)) {
	    fprintf (stderr, "cannot activate client");
	    exit(1);
    }
    npfd = snd_seq_poll_descriptors_count(seq_handle, POLLIN);
    pfd = (struct pollfd *) alloca(npfd * sizeof(struct pollfd));
    snd_seq_poll_descriptors(seq_handle, pfd, npfd, POLLIN);

    /*
    snd_midi_event_new(128, &midi_parser);
    */

    while (1) {
	if (poll(pfd, npfd, 500) > 0) {
	    midi_action(seq_handle);
	}
	//printf("pulse\n");
	/*
	snd_seq_ev_set_subs(&seq_pulse);  
	snd_seq_ev_set_direct(&seq_pulse);
	snd_midi_event_encode_byte(midi_parser, 0xF8, &seq_pulse);
	snd_seq_ev_set_source(&seq_pulse, portid_out);
	snd_seq_event_output_direct(seq_handle, &seq_pulse);
	*/
    }
}

void midi_action(snd_seq_t * seq_handle)
{

    snd_seq_event_t *ev;
    unsigned int i;

    do {
	snd_seq_event_input(seq_handle, &ev);
	switch (ev->type) {
	case SND_SEQ_EVENT_CONTROLLER:
#ifdef DEBUG
	    fprintf(stderr,
		    "Control event on Channel %2d: %5d       \r",
		    ev->data.control.channel, ev->data.control.value);
#endif
	    break;
	case SND_SEQ_EVENT_PITCHBEND:
#ifdef DEBUG
	    fprintf(stderr,
		    "Pitchbend event on Channel %2d: %5d   \r",
		    ev->data.control.channel, ev->data.control.value);
#endif
	    break;
	case SND_SEQ_EVENT_NOTEON:
	    fprintf(stderr,
		    "Selected buffer %5d (note)              \r",
		    ev->data.note.note % num_buffers);
	    for (i=0; i<num_buffers; i++) {
		    buffers[i].state = ramp_down;
	    }
	    buffers[ev->data.note.note % num_buffers].state = ramp_up;
	    break;
	case SND_SEQ_EVENT_PGMCHANGE:
	    fprintf(stderr,
		    "Selected buffer %5d (pgm change)        \r",
		    ev->data.control.value % num_buffers);
	    for (i=0; i<num_buffers; i++) {
		    buffers[i].state = ramp_down;
	    }
	    buffers[ev->data.control.value % num_buffers].state = ramp_up;
	    break;
	case SND_SEQ_EVENT_NOTEOFF:
#ifdef DEBUG
	    fprintf(stderr,
		    "Note Off event on Channel %2d: %5d      \r",
		    ev->data.control.channel, ev->data.note.note);
#endif
	    break;
	default:
#ifdef DEBUG
	    fprintf(stderr, "Unknown event\n");
#endif
	    break;
	}
	snd_seq_free_event(ev);
    } while (snd_seq_event_input_pending(seq_handle, 0) > 0);
}

/* vi:set ts=8 sts=4 sw=4: */
